@@ -1,95 +1,18 @@
 from pf2e_wealth_calculator.dataframes import itemlist, rune_replacer, tbl, materials
+from pf2e_wealth_calculator.structs import *
 
 import pandas as pd
+import numpy as np
+from tabulate import tabulate
+
 import re
 from difflib import get_close_matches
-from dataclasses import dataclass, field
 
 import sys
-import os
-import argparse
 import textwrap
-
 import typing  # for backwards compatibility to python 3.9
 
 
-class OriginError(Exception):
-    """Mismatched origin in operation between Money objects."""
-
-    def __init__(self, message):
-        super().__init__(message)
-
-
-@dataclass
-class Money:
-    """Simple data structure for cp/sp/gp amounts."""
-
-    cp: int = 0
-    sp: int = 0
-    gp: int = 0
-    origin: str = "item"
-    check_origin: bool = True
-
-    def __add__(self, val):
-        if type(val) == int:
-            return Money(
-                self.cp + val,
-                self.sp + val,
-                self.gp + val,
-                self.origin,
-                self.check_origin,
-            )
-
-        if type(val) == Money:
-            if self.check_origin and val.check_origin and self.origin != val.origin:
-                raise OriginError("Origins don't match")
-            return Money(
-                self.cp + val.cp,
-                self.sp + val.sp,
-                self.gp + val.gp,
-                self.origin,
-                self.check_origin,
-            )
-
-        raise TypeError(
-            f"Unsupported sum operation for type {type(val)} on class Money"
-        )
-
-    def __radd__(self, val):
-        return self.__add__(val)
-
-    def __mul__(self, val):
-        if type(val) == int:
-            return Money(
-                self.cp * val,
-                self.sp * val,
-                self.gp * val,
-                self.origin,
-                self.check_origin,
-            )
-
-        else:
-            raise TypeError(
-                f"Unsupported multiplication operation for type {type(val)} on class Money"
-            )
-
-    def __rmul__(self, val):
-        return self.__mul__(val)
-
-
-@dataclass(frozen=True)
-class ItemInfo:
-    """Data structure that contains information on a given item."""
-
-    name: str = "item"
-    price: Money = field(default_factory=Money)
-    category: str = "none"
-    subcategory: str = "none"
-    level: int = 0
-    rarity: str = "common"
-    bulk: typing.Union[int, str] = 0
-
-
 def get_higher_rarity(rar1: str, rar2: str) -> str:
     """Get the higher rarity among the two given ones."""
 
@@ -151,10 +74,16 @@ def parse_database(
     item_name = item_name.strip()
 
     # Check if the item name is just plain currency, in which case exit early
-    if re.match(r"\d+([,\.]\d*)?\s*(cp|sp|gp)", item_name) is not None:
-        currency_value = get_price(item_name, amount)
-        currency_value.origin = "currency"
-        return ItemInfo(item_name, currency_value, "currency", "none")
+    # The regex checks for an asterisk maybe followed by spaces, then digits, then an optional decimal
+    # comma or dot followed by more digits, followed by cp, sp or gp after any amount of spaces
+    if re.match(r"\*?\ *\d+([,\.]\d*)?\ *(cp|sp|gp)", item_name) is not None:
+        if item_name.find("*") == 0:
+            item_name = item_name.replace("*", "", 1)
+            currency_value = get_price(item_name, amount, origin=Origins.ART_OBJECT)
+            return ItemInfo(item_name, currency_value, "art objects", "none")
+        else:
+            currency_value = get_price(item_name, amount, origin=Origins.CURRENCY)
+            return ItemInfo(item_name, currency_value, "currency", "none")
 
     # Check if the first one or two words denote a precious material
     item_name, material, grade = get_material_grade(item_name.split(), materials)
@@ -170,6 +99,7 @@ def parse_database(
         )
 
     # If category is restricted, check only items from that category
+    # TODO: This can probably be simplified
     if restrict_cat:
         filtered_list = itemlist.set_index("category")
         filtered_list = filtered_list.filter(like=restrict_cat, axis=0)
@@ -210,28 +140,44 @@ def parse_database(
         item_price = get_price(item_row["price"].item(), amount)
     else:
         item_name = f"{material} {item_name} {grade}"
-        # Convert category to a code-legible string
+        # Convert category to a AoN-legible name
         categories = {
             "weapons": "weapon",
             "armor": "armor",
             "shields": "shield",
-            "materials": "object",
-            "adventuring gear": "object"
-            # TODO: Might need to add more
         }
-        category = (
-            categories[item_category] if "buckler" not in item_name else "buckler"
-        )
+
+        if "buckler" in item_name:
+            # Bucklers are priced differently than other shields
+            category = "buckler"
+        elif item_category in categories.keys():
+            category = categories[item_category]
+        else:
+            # If it's not a weapon, an armor set or a shield, it's a generic object
+            category = "object"
+
         material_name = f"{material} {category} {grade}"
+        material_row = df[df["name"] == material_name]
 
         # Add the price of the precious material
-        item_price = get_price(df[df["name"] == material_name]["price"].item(), amount)
+        item_price = get_price(material_row["price"].item(), amount)
+        # Add the extra price based on bulk
+        # Formula: price of precious item + 10% of price * Bulk (for weapons and armor)
+        #          price of precious item * Bulk (for objects)
+        #          No additions for shields and bucklers
+        if type(item_bulk) is int and category in ("weapon", "armor"):
+            item_price.gp += item_price.gp // 10 * item_bulk
+        elif category == "object":
+            multiplier = (
+                1 if type(item_bulk) is str and "L" in item_bulk else int(item_bulk)
+            )
+            item_price = item_price * multiplier if multiplier > 0 else item_price
 
         # Materials have their own level and rarity, pick the highest ones
-        material_level = df[df["name"] == material_name]["level"].item()
+        material_level = material_row["level"].item()
         item_level = material_level if material_level > item_level else item_level
 
-        material_rarity = df[df["name"] == material_name]["rarity"].item()
+        material_rarity = material_row["rarity"].item()
         item_rarity = get_higher_rarity(material_rarity, item_rarity)
 
     return ItemInfo(
@@ -410,7 +356,7 @@ def rune_calculator(
     )
 
 
-def get_price(price_str: str, amount: int = 1) -> Money:
+def get_price(price_str: str, amount: int = 1, origin: Origins = Origins.ITEM) -> Money:
     """
     Get the price of the given item.
 
@@ -431,22 +377,26 @@ def get_price(price_str: str, amount: int = 1) -> Money:
         coin_type = type_match.group()
 
         if coin_type == "cp":
-            return Money(int(item_price.replace(",", "")) * amount, 0, 0)
+            return Money(int(item_price.replace(",", "")) * amount, 0, 0, origin=origin)
         elif coin_type == "sp":
-            return Money(0, int(item_price.replace(",", "")) * amount, 0)
+            return Money(0, int(item_price.replace(",", "")) * amount, 0, origin=origin)
         elif coin_type == "gp":
-            return Money(0, 0, int(item_price.replace(",", "")) * amount)
+            return Money(0, 0, int(item_price.replace(",", "")) * amount, origin=origin)
         else:  # This shouldn't even be reachable
             raise ValueError("Invalid currency type")
     else:
         return Money()
 
 
-def console_entry_point(input_file, level, currency, detailed, noconversion):
-    """Primary entry point for the script."""
-
+def process_loot_file(filepath: str) -> pd.DataFrame:
     # User-defined loot
-    loot = pd.read_csv(input_file, names=["name", "amount"])
+    loot = pd.read_csv(filepath, names=["name", "amount"])
+    # Skip rows starting in #
+    # TODO: Skip only if # is the first character
+    for i, row in enumerate(loot["name"]):
+        if "#" in row:
+            loot.drop(i, axis=0, inplace=True)
+
     loot["name"] = loot["name"].apply(lambda name: name.lower())
     # Assume no amount means 1
     loot["amount"].fillna(1, inplace=True)
@@ -454,14 +404,18 @@ def console_entry_point(input_file, level, currency, detailed, noconversion):
     loot["amount"].replace(r"\D", 0, regex=True, inplace=True)
     loot["amount"] = loot["amount"].astype(int)
 
-    money = {"item": Money(origin="item"), "currency": Money(origin="currency")}
+    return loot
 
-    levels: dict[str, int] = {}
-    categories: dict[str, int] = {}
-    subcategories: dict[str, int] = {}
-    rarities: dict[str, int] = {}
 
-    # Get the price for each item
+def get_loot_stats(
+    loot: pd.DataFrame,
+    money: dict[Origins, Money],
+    levels: dict[str, int],
+    categories: dict[str, int],
+    subcategories: dict[str, int],
+    rarities: dict[str, int],
+):
+    """Get the price for each item in the loot DataFrame."""
     for _, row in loot.iterrows():
         name, amount = row.tolist()
         # Check if there is a fundamental rune in the item
@@ -492,45 +446,84 @@ def console_entry_point(input_file, level, currency, detailed, noconversion):
         except KeyError:
             rarities[curr_item.rarity] = amount
 
-    # Manage the level, if given
-    if level:
-        try:
-            level = [int(x) for x in level.split("-")]
-        except ValueError:
-            print(
-                "Invalid level type\nPlease only insert an integer or a range with the syntax X-Y"
-            )
-            sys.exit(1)
 
-        if len(level) == 1:
-            level = level[0]
-        elif len(level) > 2:
-            print(
-                "Invalid level type\nPlease only insert an integer or a range with the syntax X-Y"
-            )
-            sys.exit(1)
+def convert_input_level(level) -> typing.Union[int, tuple[int, int]]:
+    """Transform the user input level into a integer or a tuple of two integers."""
+    try:
+        level = [int(x) for x in level.split("-")]
+    except ValueError:
+        print(
+            "Invalid level type\nPlease only insert an integer or a range with the syntax X-Y"
+        )
+        sys.exit(1)
 
-        if type(level) is list:
-            if 0 < level[0] <= 20 and 0 < level[1] <= 20:
-                # Get total value from the TBL table
-                total_value = tbl["Total Value"][min(level) - 1 : max(level)].sum()
-            else:
-                print("Please only insert levels between 1 and 20")
-                sys.exit(1)
-        elif type(level) is int:
-            if 0 < level <= 20:
-                # Get total value from the TBL table
-                total_value = tbl.at[level - 1, "Total Value"]
-            else:
-                print("Please only insert a level between 1 and 20")
-                sys.exit(1)
+    if len(level) == 1:
+        level = level[0]
+    elif len(level) == 2:
+        level = tuple(level)
+    else:
+        print(
+            "Invalid level type\nPlease only insert an integer or a range with the syntax X-Y"
+        )
+        sys.exit(1)
+
+    return level
+
+
+def get_value_from_level(level) -> int:
+    """Find the amount of gold expected by the Treasure by Level table for the range of levels provided."""
+    if type(level) is tuple:
+        if 0 < level[0] <= 20 and 0 < level[1] <= 20:
+            total_value = tbl["Total Value"][min(level) - 1 : max(level)].sum()
         else:
-            print(
-                "Invalid level type\nPlease only insert an integer or a range with the syntax X-Y"
-            )
-            sys.exit(1)  # Probably unreachable, but type safety y'know
+            print("Please only insert levels between 1 and 20")
+            sys.exit(1)
 
-    money["currency"] += Money(gp=currency, origin="currency")
+    elif type(level) is int:
+        if 0 < level <= 20:
+            total_value = tbl.at[level - 1, "Total Value"]
+        else:
+            print("Please only insert a level between 1 and 20")
+            sys.exit(1)
+
+    else:
+        print(
+            "Invalid level type\nPlease only insert an integer or a range with the syntax X-Y"
+        )
+        sys.exit(1)  # Probably unreachable, but type safety y'know
+
+    return total_value
+
+
+def console_entry_point(
+    input_files: list[str],
+    level_str: str,
+    currency: int,
+    detailed: bool,
+    noconversion: bool,
+):
+    """Primary entry point for the script."""
+
+    money = {
+        Origins.ITEM: Money(origin=Origins.ITEM),
+        Origins.ART_OBJECT: Money(origin=Origins.ART_OBJECT),
+        Origins.CURRENCY: Money(origin=Origins.CURRENCY),
+    }
+    levels: dict[str, int] = {}
+    categories: dict[str, int] = {}
+    subcategories: dict[str, int] = {}
+    rarities: dict[str, int] = {}
+
+    for file in input_files:
+        loot = process_loot_file(file)
+
+        get_loot_stats(loot, money, levels, categories, subcategories, rarities)
+
+        if level_str:
+            level = convert_input_level(level_str)
+            total_value = get_value_from_level(level)
+
+    money[Origins.CURRENCY] += Money(gp=currency, origin=Origins.CURRENCY)
 
     # Convert coins in gp where possible, if requested
     if not noconversion:
@@ -541,12 +534,12 @@ def console_entry_point(input_file, level, currency, detailed, noconversion):
             money[origin].sp %= 10
 
     def get_total(money_list):
-        res = Money(origin="Total", check_origin=False)
+        res = Money(origin=Origins.TOTAL, check_origin=False)
         for item in money_list:
             res += item
         return res
 
-    money["total"] = get_total(money.values())
+    money[Origins.TOTAL] = get_total(money.values())
 
     print("\nTotal value", end="")
     if not noconversion:
@@ -555,26 +548,27 @@ def console_entry_point(input_file, level, currency, detailed, noconversion):
     print(
         textwrap.dedent(
             f"""\
-          - {money["total"].cp} cp
-          - {money["total"].sp} sp
-          - {money["total"].gp} gp
+          - {money[Origins.TOTAL].cp} cp
+          - {money[Origins.TOTAL].sp} sp
+          - {money[Origins.TOTAL].gp} gp
 
         Of which:
-          - Items: {money["item"].gp} gp
-          - Currency: {money["currency"].gp} gp\
+          - Items: {money[Origins.ITEM].gp} gp
+          - Art objects: {money[Origins.ART_OBJECT].gp} gp
+          - Currency: {money[Origins.CURRENCY].gp} gp\
         """
         )
     )
 
-    if level:
+    if level_str:
         print("\nDifference:")
-        if total_value - money["total"].gp < 0:
+        if total_value - money[Origins.TOTAL].gp < 0:
             print(
-                f"  - {abs(total_value - money['total'].gp)} gp too much (Expected {total_value} gp)"
+                f"  - {abs(total_value - money[Origins.TOTAL].gp)} gp too much (Expected {total_value} gp)"
             )
-        elif total_value - money["total"].gp > 0:
+        elif total_value - money[Origins.TOTAL].gp > 0:
             print(
-                f"  - {abs(total_value - money['total'].gp)} gp too little (Expected {total_value} gp)"
+                f"  - {abs(total_value - money[Origins.TOTAL].gp)} gp too little (Expected {total_value} gp)"
             )
         else:
             print(f"  - None (Expected {total_value} gp)")
@@ -598,6 +592,7 @@ def console_entry_point(input_file, level, currency, detailed, noconversion):
 
     print()
 
+
 def find_single_item(item_name: str):
     """Fetches and prints information on a single item instead of a table."""
 
@@ -630,124 +625,36 @@ def find_single_item(item_name: str):
     )
 
 
-def entry_point():
-    parser = argparse.ArgumentParser(
-        description="A simple tool for Pathfinder 2e that calculates how much your loot is worth."
-    )
-    parser.add_argument(
-        "input",
-        type=str,
-        nargs="?",
-        default="",
-        help="the name of the text file containing the loot",
-    )
-    parser.add_argument(
-        "-i",
-        "--item",
-        type=str,
-        help="run the script with only the specified item and exit",
-    )
-    parser.add_argument(
-        "-l",
-        "--level",
-        type=str,
-        help="the level of the party; can be an integer or of the form X-Y (eg. 5-8)",
-    )
-    parser.add_argument(
-        "-c",
-        "--currency",
-        type=int,
-        default=0,
-        help="a flat amount of gp to add to the total",
-    )
-    parser.add_argument(
-        "-d",
-        "--detailed",
-        action="store_true",
-        help="show more information about the items than usual"
-    )
-    parser.add_argument(
-        "-f",
-        "--format",
-        action="store_true",
-        help="show formatting instructions and exit",
-    )
-    parser.add_argument(
-        "-n",
-        "--no-conversion",
-        action="store_true",
-        help="prevent conversion of coins into gp",
-    )
-    args = parser.parse_args()
+def generate_random_items(n_of_items: int, level_str: str = "0-100"):
+    level = convert_input_level(level_str)
 
-    if args.format:
-        # Add more info in the formatting instructions
-        print(
-            textwrap.dedent(
-                """\
-            [TEXT FILE FORMAT]
-            The text file must contain two comma-separated columns:
-            The first is the item name, which is case insensitive but requires correct spelling
-            The second is the amount of items you want to add and must be a positive integer
-            This means that each row is an item name and how many there are
-            The amount can be omitted, in which case it'll default to 1
-
-            [VALID ITEM NAMES]
-            The item name must use the spelling used on the Archives of Nethys
-            If the item has a grade, it must added in brackets after the name
-            For instance, "smokestick (lesser)" is correct, "lesser smokestick" is not
-
-            The item name can also be an item with runes etched into it and the price will be calculated automatically
-            "+1 striking longsword" is a valid name, as is "+3 major striking greater shock ancestral echoing vorpal glaive"
-            For runes specifically, the grade must be placed before the rune itself, as you would write normally
-            It should be "+2 greater striking longbow" and not "+2 striking (greater) longbow"
-
-            The item can also include a precious material, though you must specify the grade
-            The grade must be after the item name and simply needs to include "low", "standard" or "high"
-            "silver dagger (low-grade)" is correct, as is "silver dagger low"
-            Make sure that it's only one word: "high-grade" is ok, "high grade" is not
-            Remember that not every material supports every grade; invalid grades currently crash the program
-
-            Runes and precious materials can be combined in one single name
-            "+1 striking mithral warhammer (standard)" is valid
-
-            The item can also be plain currency, though you still need to specify the amount
-            "32gp" is a valid item name
-            Accepted currencies are "cp", "sp" and "gp"; "pp" in not supported
-
-            [SAMPLE FILE]
-            longsword
-            oil of potency, 2
-            smokestick (lesser), 5
-            32sp
-            +1 striking shock rapier
-            storm flash
-            cold iron warhammer (standard)
-
-            [LEVEL RANGES]
-            Levels can be input both as a single value (like "1") and as a range (like "1-6")
-            A single value represents the amount of treasure that the players should find over the course of that level
-            A range instead represents the amount of treasure expected for that range of levels
-            For example, "1" is how much treasure should be given to the PCs progressing through level 1
-            while "1-3" refers to how much treasures should be given throughout levels 1, 2 and 3
-            The values are taken from the Treasure by Level table on page 508 of the Core Rulebook
-            On Archives of Nethys: https://2e.aonprd.com/Rules.aspx?ID=581\
-            """
-            )
+    if type(level) is int:
+        rand_ids = np.random.choice(
+            itemlist[itemlist["level"] == level].index, n_of_items
         )
-        sys.exit(0)
-
-    if args.item:
-        find_single_item(args.item)
-        sys.exit(0)
-
-    if os.path.isfile(args.input) and args.input.endswith(".txt"):
-        console_entry_point(args.input, args.level, args.currency, args.detailed, args.no_conversion)
-        sys.exit(0)
+        rand_items = itemlist.iloc[rand_ids]
+    elif type(level) is tuple:
+        rand_ids = np.random.choice(
+            itemlist[
+                (itemlist["level"] >= level[0]) & (itemlist["level"] <= level[1])
+            ].index,
+            n_of_items,
+        )
+        rand_items = itemlist.iloc[rand_ids]
     else:
-        print("Please input a valid text file or use the -i option")
         sys.exit(1)
 
+    rand_items = rand_items.applymap(
+        lambda word: word.capitalize() if type(word) is str else word
+    )
 
-if __name__ == "__main__":
-    entry_point()
+    print()
+    print(
+        tabulate(
+            rand_items,  # type: ignore
+            headers=rand_items.columns.str.capitalize(),  # type: ignore
+            showindex=False,
+            tablefmt="rounded_outline",
+        )
+    )
+    print()
